/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.command.CommandTestingApplication.java
 * Created on: Jun 12, 2016
 */
package net.bigeon.gclc;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;

import net.bigeon.gclc.command.ICommand;
import net.bigeon.gclc.exception.InvalidCommandName;
import net.bigeon.gclc.utils.PipedConsoleInput;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings("javadoc")
public class CommandTestingApplication implements AutoCloseable {

    private final ConsoleApplication application;
    private final Thread             th;
    private final PipedConsoleOutput out;
    private final PipedConsoleInput  in;

    /** @throws IOException if the streams cannot be build */
    public CommandTestingApplication() throws IOException {
        out = new PipedConsoleOutput();
        in = new PipedConsoleInput(null);
        application = new ConsoleApplication(out, in, "", "");
        new ConsoleTestApplication().attach(application);
        th = new Thread(new Runnable() {

            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                application.start();
            }
        });

        th.start();
    }

    /** @param cmd the command
     * @return if the command was added
     * @throws InvalidCommandName if the command name is invalid
     * @see net.bigeon.gclc.ConsoleApplication#add(net.bigeon.gclc.command.ICommand) */
    public final boolean add(final ICommand cmd) throws InvalidCommandName {
        return application.add(cmd);
    }

    @Override
    public void close() throws IOException {
        application.exit();
        out.close();
        in.close();
    }

    /** @return the application */
    public ConsoleApplication getApplication() {
        return application;
    }

    /** @return the next written line, null if it is the prompt
     * @throws IOException if the reading failed */
    public String readNextLine() throws IOException {
        final String ret = out.readNextLine();
        return ret;
    }

    public void sendCommand(final String cmd) throws IOException {
        in.type(cmd);
    }

    /** @throws IOException if the writing failed */
    public void waitAndStop() throws IOException {
        in.type(ConsoleTestApplication.EXIT);
        try {
            th.join();
        } catch (final InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

}
